"""
interval
~~~~~~~~

Utility functions and classes used in applications of Interval Design.

:copyright: (c) 2022 by Interval Design.
:license: MIT, see LICENSE for more details.
"""

from . import ddd
from . import utils
